Package: conflicted
Title: An Alternative Conflict Resolution Strategy
Version: 1.2.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: R's default conflict management system gives the most
    recently loaded package precedence. This can make it hard to detect
    conflicts, particularly when they arise because a package update
    creates ambiguity that did not previously exist. 'conflicted' takes a
    different approach, making every conflict an error and forcing you to
    choose which function to use.
License: MIT + file LICENSE
URL: https://conflicted.r-lib.org/, https://github.com/r-lib/conflicted
BugReports: https://github.com/r-lib/conflicted/issues
Depends: R (>= 3.2)
Imports: cli (>= 3.4.0), memoise, rlang (>= 1.0.0)
Suggests: callr, covr, dplyr, Matrix, methods, pkgload, testthat (>=
        3.0.0), withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-01-31 19:50:23 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  RStudio [cph, fnd]
Maintainer: Hadley Wickham <hadley@rstudio.com>
Repository: CRAN
Date/Publication: 2023-02-01 08:20:06 UTC
Built: R 4.6.0; ; 2026-04-26 02:30:17 UTC; unix
