# Generated by roxygen2: do not edit by hand

S3method("[",conflict_report)
S3method(print,conflict_report)
export(conflict_prefer)
export(conflict_prefer_all)
export(conflict_prefer_matching)
export(conflict_scout)
export(conflicts_prefer)
import(rlang)
