#' @section Resolving conflicts:
#'
#' To permanently resolve a conflict within a session, use assignment:
#'
#' \preformatted{
#' library(conflicted)
#' library(dplyr)
#'
#' filter <- dplyr::filter
#' }
#'
#' @keywords internal
#' @import rlang
"_PACKAGE"
